@@ -375,11 +375,12 @@ class NREN
 	{
 		if (!is_null($url)) {
 			if ($this->data['wayf_url'] != $url) {
-				if (!preg_match("/^http[s]?/",$url, $matches)) {
+				if (empty($url) || preg_match("/^http[s]?/",$url, $matches)) {
+					$this->data['wayf_url'] = Input::sanitizeURL($url);
+					$this->pendingChanges = true;
+				} else {
 					return false;
 				}
-				$this->data['wayf_url'] = Input::sanitizeURL($url);
-				$this->pendingChanges = true;
 			}
 		}
 		return true;
@@ -418,7 +419,7 @@ class NREN
 	{
 		if (!is_null($url)) {
 			if ($this->data['url'] != $url) {
-				$this->data['url'] = $url;
+				$this->data['url'] = Input::sanitizeURL($url);
 				$this->pendingChanges = true;
 			}
 		}
@@ -440,7 +441,7 @@ class NREN
 	{
 		if (!is_null($contact_email)) {
 			if ($this->data['contact_email'] != $contact_email) {
-				$this->data['contact_email'] = $contact_email;
+				$this->data['contact_email'] = Input::sanitizeEmail($contact_email);
 				$this->pendingChanges = true;
 			}
 		}
@@ -456,7 +457,7 @@ class NREN
 	{
 		if (!is_null($contact_phone)) {
 			if ($this->data['contact_phone'] != $contact_phone) {
-				$this->data['contact_phone'] = $contact_phone;
+				$this->data['contact_phone'] = Input::sanitizePhone($contact_phone);
 				$this->pendingChanges = true;
 			}
 		}
@@ -473,7 +474,7 @@ class NREN
 	{
 		if (!is_null($cert_email)) {
 			if ($this->data['cert_email'] != $cert_email) {
-				$this->data['cert_email'] = $cert_email;
+				$this->data['cert_email'] = Input::sanitizeEmail($cert_email);
 				$this->pendingChanges = true;
 			}
 		}
@@ -733,6 +734,14 @@ class NREN
 					$key = strtolower(Input::sanitizeText($k));
 					$this->data[$key] = Input::sanitizeText($value);
 				}
+
+				/* hack to work around MySQLs very limited view on the difference
+				 * between 0 and NULL
+				 */
+				if (is_null($this->data['enable_email'])) {
+					$this->data['enable_email'] = 0;
+				}
+
 				break;
 			default:
 				echo "too many nrens (" . count($res) . ") found in db. Aborting.<br />\n";
