@@ -54,21 +54,19 @@ class Subscriber
 		}
 
 		/* ugly hack to circumvent the missing constructor overloading of PHP5 */
+		$this->nren = $nren; /* NREN-object */
+		$this->idp_name = Input::sanitizeIdPName($idp_name);
 		if (isset($dn_name) && isset($org_state)) {
-			$this->nren = $nren;
-			$this->idp_name = $idp_name;
-			$this->org_name = $dn_name;
-			$this->state = $org_state;
-			$this->db_id = $db_id;
+			$this->setOrgName($dn_name);
+			$this->setState($org_state);
+			$this->setDBID($db_id);
 		} else {
-			$this->nren	= $nren;
-			$this->idp_name = trim($idp_name);
 			$this->valid	= $this->updateFromDB();
 			if ($this->valid) {
 				$this->retrieveMap();
 			}
 		}
-	}
+	} /* end __construct() */
 
 	/**
 	 * toString() Return a plain textlist (html-formatted) of a subscriber
@@ -151,7 +149,7 @@ class Subscriber
 	 */
 	public function getOrgName()
 	{
-		$res = $this->org_name;
+		$res = stripslashes($this->org_name);
 		/**
 		 * set the test prefix, if confusa is in 'capi_test' mode
 		 */
@@ -168,7 +166,11 @@ class Subscriber
 			if ($org_name === $this->org_name) {
 				return false;
 			}
-			$this->org_name = Input::sanitizeText($org_name);
+			If (Config::get_config('cert_product') == PRD_ESCIENCE) {
+				$this->org_name = Input::sanitizeOrgName($org_name);
+			} else {
+				$this->org_name = mysql_real_escape_string($org_name);
+			}
 			return true;
 		}
 		return false;
@@ -208,13 +210,13 @@ class Subscriber
 					"there are uncommited messages in Subscriber";
 			}
 		}
-		$query = "SELECT * FROM subscribers WHERE name = ? AND nren_id = ?";
+		$query = "SELECT * FROM subscribers WHERE name=:subscriber_name AND nren_id=:nren_id";
+		$data = array();
+		$data['subscriber_name'] = $this->idp_name;
+		$data['nren_id'] = $this->nren->getID();
 		try {
-			$res = MDB2Wrapper::execute($query,
-						    array('text', 'text'),
-						    array($this->idp_name, $this->nren->getID()));
+			$res = MDB2Wrapper::execute($query, null, $data);
 			if (count($res) != 1) {
-				/* Could not find the subscriber. Aborting. */
 				return false;
 			}
 		} catch (DBStatementException $dbse) {
