@@ -42,9 +42,9 @@ class CA_Comodo extends CA
 		}
 
 		if (Config::get_config('cert_product') == PRD_ESCIENCE) {
-			$this->dcs[] = "tcs";
-			$this->dcs[] = "terena";
 			$this->dcs[] = "org";
+			$this->dcs[] = "terena";
+			$this->dcs[] = "tcs";
 		}
     }
 
@@ -114,33 +114,6 @@ class CA_Comodo extends CA
                                 $encrypted_pw, MCRYPT_MODE_CFB,
                                 $ivector)));
     }
-
-    /**
-     * Lookup a list of user certificates from cache
-     * The cache is tied to a user simplesamlphp session
-     *
-     * @return the list of unprocessed certificates as they were received from
-     *         the online CA
-     */
-    private function cacheLookupList()
-    {
-		return CS::getSessionKey('rawCertList');
-    }
-
-    /**
-     * Insert a list of user certificates into the cache
-     *
-     * @param $raw_list the (unprocessed) array of certificates as they were
-     *        received
-     * @param $days integer the number of days of certificate history that is
-     *                      included in raw_list
-     */
-    private function cacheInsertList($raw_list, $days)
-    {
-		CS::setSessionKey('rawCertList', $raw_list);
-		CS::setSessionKey('confusaCachedDays', $days);
-    } /* end cacheInsertList */
-
 	/**
 	 * Set an expiry date on the cache based on the time of the latest
 	 * certificate order. The idea is that the more recently a certificate has
@@ -237,94 +210,54 @@ class CA_Comodo extends CA
 		}
 	}
 
-    /**
-     * Delete the certificate list from cache. Useful if there were changes
-     * (Revocation, insertion)
-     */
-    private function cacheInvalidate()
-    {
-		CS::deleteSessionKey('rawCertList');
-    }
 
 	/**
 	 * Sign the CSR identified by auth_key using the Online-CA's remote API
-	 * @throws ConfusaGenException
-	*/
-	public function signKey($auth_key, $csr)
+	 *
+	 * @param	String the auth-key used to identify the CSR in the database
+	 * @param	CSR the CSR to be signed
+	 * @return	void
+	 * @access	public
+	 *
+	 * @fixme	make sure all callers of signKey is updated to use CSR.
+	 */
+	public function signKey($csr)
 	{
 		if (!$this->person->getSubscriber()->isSubscribed()) {
 			throw new KeySignException("Subscriber not subscribed, cannot create certificate!");
 		}
-		$this->capiUploadCSR($auth_key, $csr);
+
+		$authKey = $csr->getAuthToken();
+
+		Logger::logEvent(LOG_DEBUG, __CLASS__, "signKey()",
+		                 "Signing CSR with authKey $authKey",
+		                 __LINE__);
+
+		/* FIXME: better solution */
+		if ($csr instanceof CSR_PKCS10) {
+			$this->capiUploadCSR($authKey,
+			                     $csr->getPEMContent(),
+			                     ConfusaConstants::$CAPI_FORMAT_PKCS10);
+		} else if ($csr instanceof CSR_SPKAC) {
+			$this->capiUploadCSR($authKey,
+			                     $csr->getDERContent(),
+			                     ConfusaConstants::$CAPI_FORMAT_SPKAC);
+		}
+
 		$this->capiAuthorizeCSR();
 
-		$this->cacheInvalidate();
+		CS::deleteSessionKey('rawCertList');
 		$timezone = new DateTimeZone($this->person->getTimezone());
-		$dt = new DateTime("now", $timezone);
+		$dt       = new DateTime("now", $timezone);
 
 		CA::sendMailNotification($this->order_number,
 		                         $dt->format('Y-m-d H:i T'),
 		                         $_SERVER['REMOTE_ADDR'],
 		                         $this->person,
 		                         $this->getFullDN());
-	/* FIXME: conflict, not sure how to resolve, do we need both? */
-        Logger::log_event(LOG_INFO, "Signed CSR for user with auth_key $auth_key");
-	/* FIXME: <END> */
-    }
-
-    /**
-     * Sign a browser generated CSR, generated by the specified browser
-     *
-     * The handling only differs in the format, which is PKCS10/CSR for IE,
-     * SPKAC for keygen-enabled browsers and CMRF for Firefox.
-     *
-     * @param $csr the CSR
-     * @param $browser the browser with which it was generated
-     *
-     * @return the order_number of the certificate, so its status can be
-     * polled from the graphical interface
-     */
-    public function signBrowserCSR($csr, $browser)
-    {
-	    if (!$this->person->getSubscriber()->isSubscribed()) {
-		    throw new KeySignException("Subscriber not subscribed, cannot create certificate!");
-	    }
-        /* use the last 64-characters of the CRMF as an auth_key */
-		$auth_key = substr($csr, strlen($csr)-65, strlen($csr)-1);
-        switch($browser) {
-        case "msie_post_vista":
-            $this->capiUploadCSR($auth_key, $csr, 'csr');
-            break;
-
-        case "msie_pre_vista":
-            $this->capiUploadCSR($auth_key, $csr, 'csr');
-            break;
-
-        case "keygen":
-            $this->capiUploadCSR($auth_key, $csr, 'spkac');
-            break;
-
-        default:
-            throw new ConfusaGenException("Browser $browser is unsupported!");
-            break;
-        }
-
-		$this->capiAuthorizeCSR();
-		$this->cacheInvalidate();
-
-		$timezone = new DateTimeZone($this->person->getTimezone());
-		$dt = new DateTime("now", $timezone);
-
-		CA::sendMailNotification($this->order_number,
-		                         $dt->format('Y-m-d H:i T'),
-		                         $_SERVER['REMOTE_ADDR'],
-		                         $this->person,
-								 $this->getFullDN());
-
-		Logger::log_event(LOG_INFO, "Signed CSR for user with order_number " .
-		                            $this->order_number);
-        return $this->order_number;
-    }
+		Logger::log_event(LOG_INFO, "Signed CSR for user with authKey $authKey");
+		return $this->order_number;
+	} /* end signKey() */
 
     /**
      * Return an array with all the certificates obtained by the person managed by this
@@ -355,7 +288,7 @@ class CA_Comodo extends CA
 		 * "Certificate" class
 		 */
 		if ($this->cacheHasCertHistory($days)) {
-			$res = $this->cacheLookupList();
+			$res = CS::getSessionKey('rawCertList');
 
 			if (isset($res)) {
 				/* apply local date filtering (much faster than querying again) */
@@ -418,7 +351,7 @@ class CA_Comodo extends CA
 			}
 
             $res[$i-1]['order_number'] = $params[$i . '_orderNumber'];
-            $res[$i-1]['cert_owner'] = $this->person->getX509ValidCN();
+            $res[$i-1]['cert_owner'] = stripslashes($this->person->getX509ValidCN());
 			$res[$i-1]['status'] = $status;
 
 			if (isset($params[$i . '_1_notBefore'])) {
@@ -431,9 +364,11 @@ class CA_Comodo extends CA
         }
 
 		$this->cacheSetExpiryDate(min($dates));
-		$this->cacheInsertList($res, $days);
+		CS::setSessionKey('rawCertList', $res);
+		CS::setSessionKey('confusaCachedDays', $days);
         return $res;
     }
+
     /* delete a certificate from the DB (Deprecated)
      *
      * May come in handy when we have the cache for online-certificates though.
@@ -645,10 +580,12 @@ class CA_Comodo extends CA
     /**
      * Retrieve a certificate from a remote endpoint (e.g. Comodo).
      *
-     * @params key either an order-number that can be used to retrieve a certificate
-     * directly or an auth-key with which we can retrieve the order-number
-     *
-     * @param $key The order-number or an auth_key that can be transformed to order_number
+     * @params	key	either an order-number that can be used to retrieve a
+     *			certificate directly or an auth-key with which we can
+     *			retrieve the order-number
+     * @param	key	The order-number or an auth_key that can be transformed
+     *			to order_number.
+     * @access	public
      * @throws ConfusaGenException
      */
     public function getCert($key)
@@ -675,9 +612,10 @@ class CA_Comodo extends CA
          */
 
         $status=substr($data,0,1);
+	$cert = false;
         switch($status) {
         case $STATUS_OK:
-            $return_res = substr($data,2);
+		$cert = new Certificate(substr($data,2));
             break;
         case $STATUS_PEND:
             Framework::message_output("The certificate is being processed and is not yet available");
@@ -696,7 +634,7 @@ class CA_Comodo extends CA
               throw new CGE_ComodoAPIException($msg);
             }
         }
-        return $return_res;
+        return $cert;
     }
 
     /*
@@ -753,7 +691,7 @@ class CA_Comodo extends CA
 
 			switch($error_parts[0]) {
 			case $STATUS_OK:
-				$this->cacheInvalidate();
+				CS::deleteSessionKey('rawCertList');
 				Logger::log_event(LOG_NOTICE, "Revoked certificate with " .
 								  "order number $key using Comodo's AutoRevoke " .
 								  "API. User contacted us from " .
@@ -871,7 +809,24 @@ class CA_Comodo extends CA
             return "<script type=\"text/javascript\">$data</script>";
             break;
 
-        case "keygen":
+        case "chrome":
+			 $collect_endpoint = ConfusaConstants::$CAPI_COLLECT_ENDPOINT .
+                                   "?loginName=" . $this->login_name .
+                                    "&loginPassword=" . $this->login_pw .
+                                    "&orderNumber=" . $key .
+                                    "&queryType=2" .
+                                    "&responseType=3" . /* PKCS#7 */
+                                    "&responseEncoding=0"; /* encode base-64 */
+
+            $data = CurlWrapper::curlContact($collect_endpoint);
+            $cert = new Certificate(trim(substr($data, 2)));
+            $der_cert = $cert->getDERContent(true);
+            return $der_cert;
+            break;
+
+        case "mozilla":
+        case "safari":
+        case "opera":
             $collect_endpoint = ConfusaConstants::$CAPI_COLLECT_ENDPOINT .
                                    "?loginName=" . $this->login_name .
                                     "&loginPassword=" . $this->login_pw .
@@ -1018,26 +973,24 @@ class CA_Comodo extends CA
      * @param $csr_format Exactly one of "csr" (PKCS10), "crmf" or "spkac"
      *
      * @throws ConfusaGenException
+     *
+     * @fixme: Update this to use CSR (which reduce the number of arguments to 1)
     */
-    private function capiUploadCSR($auth_key, $csr, $csr_format = "csr")
+    private function capiUploadCSR($auth_key, $csr, $csr_format)
     {
         $sign_endpoint = ConfusaConstants::$CAPI_APPLY_ENDPOINT;
-		$orgName = "";
 
 		if (Config::get_config('cert_product') == PRD_PERSONAL) {
 			$ca_cert_id = ConfusaConstants::$CAPI_PERSONAL_ID;
-			$orgName = $this->person->getSubscriber()->getOrgName();
-			/* personal certificates should only have the name of the person
-			 * in the CN */
 		} else if (Config::get_config('cert_product') == PRD_ESCIENCE) {
 			$ca_cert_id = ConfusaConstants::$CAPI_ESCIENCE_ID;
-			$orgName = Output::mapUTF8ToASCII($this->person->getSubscriber()->getOrgName());
 		} else {
 			throw new KeySignException("Confusa's configured product-mode is " .
 			                           "illegal! Must be one of: PRD_ESCIENCE, " .
 			                           "PRD_PERSONAL. Please contact an IT " .
 			                           "administrator about that!");
 		}
+		$orgName = $this->person->getSubscriber()->getOrgName();
 
         $postfields_sign_req=array();
 	$pf_counter = 1;
@@ -1099,16 +1052,22 @@ class CA_Comodo extends CA
 
 		/* manually compose the subject. Necessary, because we want to have
          * Terena domainComponents */
-        $postfields_sign_req["subject_commonName_$pf_counter"] = $this->person->getX509ValidCN();
+        $postfields_sign_req["subject_commonName_$pf_counter"] =
+		stripslashes($this->person->getX509ValidCN());
 		$pf_counter++;
 
         $postfields_sign_req["subject_organizationName_".$pf_counter++] = $orgName;
         $postfields_sign_req["subject_countryName_".$pf_counter++] =
 		$this->person->getNREN()->getCountry();
 
-		foreach($this->dcs as $dc) {
-			$postfields_sign_req["subject_domainComponent_".$pf_counter++] = $dc;
-		}
+	foreach(array_reverse($this->dcs) as $dc) {
+		if ($dc == ConfusaConstants::$CAPI_TEST_DC_PREFIX)
+			continue;
+		$postfields_sign_req["subject_domainComponent_".$pf_counter++] = $dc;
+	}
+	if (Config::get_config('capi_test') == true) {
+		$postfields_sign_req["subject_domainComponent_".$pf_counter++] = ConfusaConstants::$CAPI_TEST_DC_PREFIX;
+	}
 
 	$data = CurlWrapper::curlContact($sign_endpoint, "post", $postfields_sign_req);
 
@@ -1126,7 +1085,7 @@ class CA_Comodo extends CA
 		if (isset($params['errorItem'])) {
 			$msg .= " " . $params['errorItem'];
 		}
-		$this->capiErrorMessage($params['errorCode'], $params['errorMessage']);
+		$msg .= $this->capiErrorMessage($params['errorCode'], $params['errorMessage']);
 		throw new CGE_ComodoAPIException($msg);
         }
 
@@ -1144,7 +1103,7 @@ class CA_Comodo extends CA
                                         "order number " .
                                         $this->order_number .
                                         " for user " .
-                                        $this->person->getX509ValidCN() .
+                                        stripslashes($this->person->getX509ValidCN()) .
                                         " Person contacted us from " .
                                         $_SERVER['REMOTE_ADDR']);
 
@@ -1192,7 +1151,7 @@ class CA_Comodo extends CA
 			break;
 		case "-13":
 			$msg .= "<br /><br />You created a certificate with a non-standard keysize! Please ";
-			$msg .= "create your certificate requests with a keysize of " . Config::get_config('key_length');
+			$msg .= "create your certificate requests with a keysize of " . Config::get_config('default_key_length');
 			$msg .= " bits!";
 			break;
 		case "-20":
@@ -1264,7 +1223,7 @@ class CA_Comodo extends CA
                               array('text'),
                               array($this->order_number));
           Logger::log_event(LOG_NOTICE, "Authorized remote certificate for person ".
-                                        $this->person->getX509ValidCN().
+                                        stripslashes($this->person->getX509ValidCN()).
                                         " with order number " .
                                         $this->order_number .
                                         " Person contacted us from ".
@@ -1278,6 +1237,5 @@ class CA_Comodo extends CA
         }
 
     } /* end capiAuthorizeCSR */
-
 } /* end class CA_Comodo */
 ?>
