@@ -43,7 +43,7 @@ class Input
 	 */
 	static function sanitizeOrgName($input)
 	{
-		$output = preg_replace('/[^a-z0-9@_,\.\s]/i', '', $input);
+		$output = preg_replace('/[^a-z0-9@_\-,\.\s]/i', '', $input);
 		return $output;
 	}
 
@@ -57,7 +57,7 @@ class Input
 	 */
 	static function sanitizeAntiCSRFToken($token)
 	{
-		return preg_replace('/[^0-9]+[:][^a-f0-9]/i', '', $token);
+		return preg_replace('/[^0-9]+[:][^a-f0-9]+/i', '', $token);
 	}
 
 	/**
@@ -104,8 +104,8 @@ class Input
 	/**
 	 * Sanitize an e-mail address. No validation, just dropping unwanted
 	 * characters.
-	 * @param $input string the unsanitized e-mail address
-	 * @return string the sanitized e-mail address
+	 * @param String $input the unsanitized e-mail address(es)
+	 * @return String|Array the sanitized e-mail address(es)
 	 */
 	static function sanitizeEmail($input)
 	{
@@ -143,8 +143,7 @@ class Input
 	 */
 	static function sanitizePersonName($input)
 	{
-		$output = preg_replace('|--|i', '-', $input);
-		$output = mb_ereg_replace('[^[:alpha:]\-\s\.]', '', $output, 'ip');
+		$output = mb_ereg_replace('[^[:alpha:]\-\s\.,]', '', $input, 'ip');
 		return $output;
 	}
 
@@ -174,12 +173,12 @@ class Input
 	static function sanitizeURL($input)
 	{
 		$output = preg_replace('|[^\:/\.a-z0-9\-\?&%\=~_]|i', '', $input);
-		/* strip excessive //'s*/
+		$output = preg_replace('|\://[/]*|i', '://', $output);
 		$pos = strpos($output, "://");
 		if ($pos > 0) {
 			$pos += strlen("://");
 			$output = substr($output, 0, $pos) .
-				preg_replace("/\/\//","/", substr($output, $pos));
+				preg_replace("/[\/]+/","/", substr($output, $pos));
 		}
 		return $output;
 	}
@@ -269,8 +268,7 @@ class Input
 	 */
 	static function sanitizeNumeric($input)
 	{
-		$output = preg_replace('/[^0-9]/', '', $input);
-		return $output;
+		return filter_var((int)$input, FILTER_SANITIZE_NUMBER_INT);
 	}
 
 	/**
@@ -369,5 +367,42 @@ class Input
 		$output = strtr($input, array("<br />" => "\n"));
 		return $output;
 	}
+
+	/**
+	 * Return all the characters that are included in unsanitizedInput, but
+	 * not in sanitizedInput. This is a handy function to display which
+	 * characters were sanitized away to the user. Note, that this function
+	 * can *not* be used to obtain a diff, because it only checks which
+	 * characters are in the first string, but not in the second.
+	 *
+	 * N.B: Use only on short strings!
+	 *
+	 * @param $unsanitizedInput original string
+	 * @param $sanitizedInput string that was sanitized
+	 * @return Characters that were in the original string, but not in the
+	 *         sanitized one.
+	 */
+	static function findSanitizedCharacters($unsanitizedInput,
+	                                        $sanitizedInput)
+	{
+		if (!isset($unsanitizedInput) || !isset($sanitizedInput)) {
+			return "";
+		}
+
+		if (strlen($unsanitizedInput) == 0) {
+			return "";
+		}
+
+		$returnString = "";
+
+		for ($i = 0; $i < strlen($unsanitizedInput); $i++) {
+			$currentOriginalChar = substr($unsanitizedInput, $i, 1);
+
+			if (strpos($sanitizedInput, $currentOriginalChar) === FALSE) {
+				$returnString .= $currentOriginalChar;
+			}
+		}
+		return $returnString;
+	}
 }
 ?>
